/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package settings

import (
	"sort"
	"strings"

	corev1 "k8s.io/api/core/v1"

	"k8s.libre.sh/application/settings/parameters"
	"k8s.libre.sh/interfaces"
	"k8s.libre.sh/meta"
	"k8s.libre.sh/objects"
	"k8s.libre.sh/objects/configmap"
	"k8s.libre.sh/objects/secret"
)

const (
	// Generate settings as env variables in a configMap or Secret
	GenEnvFile SettingsGenerate = "envFile"
)

const (
	// Generate settings as env variables in a configMap or Secret
	SecretSettings SettingsType = "secret"
)

type SettingsGenerate string
type SettingsType string

// +kubebuilder:object:generate=true
type Component struct {
	CreateOptions *CreateOptions `json:"createOptions,omitempty"`
	*ConfigSpec   `json:",inline"`
}

// ConfigSpec defines a list of parameters and references to resources from which those parameters can be fetched
// Only secrets and configmaps in the same namespace are supported as references
// +kubebuilder:object:generate=true
type SettingsSpec struct {
	Sources []Source `json:"sources,omitempty"`
	// Parameters is a list of parameters
	*parameters.Parameters `json:"parameters,omitempty"`
}

// +kubebuilder:object:generate=true
type CreateOptions struct {
	CommonMeta *meta.ObjectMeta `json:"commonMeta,omitempty"`
	SecretMeta *meta.ObjectMeta `json:"secretMeta,omitempty"`
	ConfigMeta *meta.ObjectMeta `json:"configMeta,omitempty"`
	//	DefaultType      parameters.ParameterType `json:"defaultType,omitempty"`
	//	DefaultMountType parameters.MountType     `json:"defaultMountType,omitempty"`
	//	Template             string    json:"template,omitempty"`
	Generate     SettingsGenerate `json:"generate,omitempty"`
	SettingsType SettingsType     `json:"type,omitempty"`
}

func (opts *CreateOptions) Init() {
	if opts.CommonMeta == nil {
		opts.CommonMeta = &meta.ObjectMeta{}
	}

	if opts.CommonMeta.Labels == nil {
		opts.CommonMeta.Labels = make(map[string]string)
	}

	if opts.SecretMeta == nil {
		opts.SecretMeta = &meta.ObjectMeta{}
	}

	if opts.SecretMeta.Labels == nil {
		opts.SecretMeta.Labels = make(map[string]string)
	}

	if opts.ConfigMeta == nil {
		opts.ConfigMeta = &meta.ObjectMeta{}
	}

	if opts.ConfigMeta.Labels == nil {
		opts.SecretMeta.Labels = make(map[string]string)
	}

}

func (s *Component) GetCreateOptions() *CreateOptions { return s.CreateOptions }

func (s *Component) GetConfig() Config { return s.ConfigSpec }

func (s *Component) GetMeta() meta.Instance { return s.CreateOptions.CommonMeta }

func (s *Component) GetObjects() map[int]objects.Object {

	cm := &configmap.ConfigMap{
		ObjectMeta: s.CreateOptions.ConfigMeta,
	}

	secret := &secret.Secret{
		ObjectMeta: s.CreateOptions.SecretMeta,
	}

	objs := make(map[int]objects.Object, 2)

	genConfigParams := []string{}
	genSecretParams := []string{}

	for _, p := range *s.ConfigSpec.Parameters {
		// TODO TO FIX
		if (p.MountType == parameters.MountEnvFile || p.MountType == parameters.MountFile) &&
			// TODO TO FIX
			(p.Type == parameters.SecretParameter || p.Type == "") &&
			len(p.Value) > 0 {

			if p.Generate != parameters.GenerateTemplate && p.Generate != "" {
				genSecretParams = append(genSecretParams, p.Key)
			}
			secret.Parameters = append(secret.Parameters, p)
		}
		if (p.MountType == parameters.MountEnvFile || p.MountType == parameters.MountFile) &&
			p.Type == parameters.ConfigParameter &&
			len(p.Value) > 0 {
			if p.Generate != parameters.GenerateTemplate && p.Generate != "" {
				genConfigParams = append(genConfigParams, p.Key)
			}
			cm.Parameters = append(cm.Parameters, p)
		}
	}

	// Sort annotations as it is sorted in kubernetes and will have a diff otherwise
	sort.Strings(genConfigParams)
	sort.Strings(genSecretParams)

	// TODO TO FIX remove annotation if secret is not generated anymore
	if len(genSecretParams) > 0 {
		if len(secret.ObjectMeta.Annotations) == 0 {
			secret.ObjectMeta.Annotations = make(map[string]string)
		}
		secret.ObjectMeta.Annotations["settings.k8s.libre.sh/generate"] = strings.Join(genSecretParams, ",")

	}

	if len(genConfigParams) > 0 {
		if len(secret.ObjectMeta.Annotations) == 0 {
			cm.ObjectMeta.Annotations = make(map[string]string)
		}
		secret.ObjectMeta.Annotations["settings.k8s.libre.sh/generate"] = strings.Join(genConfigParams, ",")
	}

	if len(secret.Parameters) > 0 {
		objs[0] = secret
	}

	if len(cm.Parameters) > 0 {
		objs[1] = cm
	}

	return objs
}

func (s *Component) SetDefaults() {

}

func (s *Component) Init(r interfaces.Reconcile, owner interfaces.Object) error {

	s.CreateOptions.Init()

	meta.SetObjectMeta(s.CreateOptions.CommonMeta, s.CreateOptions.ConfigMeta)
	meta.SetObjectMeta(s.CreateOptions.CommonMeta, s.CreateOptions.SecretMeta)

	err := InitParametersValueFrom(s, r, owner)

	if err != nil {
		return err
	}

	s.InitRandValues()

	if err != nil {
		return err
	}

	return nil
}

// InitParametersValueFrom intialise the parameters with values provided in external resources in the same namespace
// All parameters values are filled from those resources
// Only Secrets and Configmaps are supported
func InitParametersValueFrom(s *Component, r interfaces.Reconcile, owner interfaces.Object) error {

	params := parameters.Parameters{}
	cm := &corev1.ConfigMap{}
	sec := &corev1.Secret{}

	paramsBySecretSource, paramsByConfigMapSource := OrderByResourceRef(s.ConfigSpec)

	for k, v := range paramsBySecretSource {
		sec.SetNamespace(s.CreateOptions.CommonMeta.GetNamespace())
		sec.SetName(k)

		ps, err := parameters.MergeParametersFromResource(r, sec, owner, v)

		if err != nil {
			return err
		}

		params = append(params, ps...)

	}

	for k, v := range paramsByConfigMapSource {
		cm.SetName(k)
		cm.SetNamespace(s.CreateOptions.CommonMeta.GetNamespace())

		ps, err := parameters.MergeParametersFromResource(r, cm, owner, v)

		if err != nil {
			return err
		}

		params = append(params, ps...)

	}

	secretSrc := Source{
		Ref:  s.GetCreateOptions().SecretMeta.GetName(),
		Type: "secret",
	}
	configSrc := Source{
		Ref:  s.GetCreateOptions().ConfigMeta.GetName(),
		Type: "configmap",
	}

	// Reset sources, we do not want to mount orginal resources
	if s.CreateOptions.Generate == GenEnvFile {
		s.Sources = []Source{}
	}

	s.Parameters = parameters.MergeParameters(s.Parameters, &params)

	for _, v := range *s.Parameters {
		if v.MountType == parameters.MountEnvFile || s.CreateOptions.Generate == GenEnvFile {
			v.MountType = parameters.MountEnvFile
			// add newly created resource ref to sources
			if v.Type == parameters.ConfigParameter {
				s.Sources = AppendSourceIfUnique(s.Sources, configSrc)
			} else if v.Type == parameters.SecretParameter || v.Type == "" {
				s.Sources = AppendSourceIfUnique(s.Sources, secretSrc)
			}
			// Reset ref, we do not want to mount orginal resources
			v.Ref = ""
		}

		if s.CreateOptions.SettingsType == "configmap" {
			v.Type = parameters.ConfigParameter
		} else if s.CreateOptions.SettingsType == "secret" {
			v.Type = parameters.SecretParameter
		}

		if v.MountType == parameters.MountFile && len(v.Ref) == 0 {
			if v.Type == parameters.ConfigParameter {
				v.Ref = s.GetCreateOptions().ConfigMeta.GetName()

			}
			if v.Type == parameters.SecretParameter {
				v.Ref = s.GetCreateOptions().SecretMeta.GetName()

			}
		}
	}

	return nil
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SettingsSpec) DeepCopyInto(out *SettingsSpec) {
	*out = *in
	if in.Sources != nil {
		in, out := &in.Sources, &out.Sources
		*out = make([]Source, len(*in))
		copy(*out, *in)
	}
	if in.Parameters != nil {
		in, out := &in.Parameters, &out.Parameters
		*out = new(parameters.Parameters)
		if **in != nil {
			in, out := *in, *out
			*out = make([]*parameters.Parameter, len(*in))
			for i := range *in {
				if (*in)[i] != nil {
					in, out := &(*in)[i], &(*out)[i]
					*out = new(parameters.Parameter)
					**out = **in
				}
			}
		}
	}
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SettingsSpec.
func (in *SettingsSpec) DeepCopy() *SettingsSpec {
	if in == nil {
		return nil
	}
	out := new(SettingsSpec)
	in.DeepCopyInto(out)
	return out
}
