/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package parameters

import (
	"context"
	"errors"
	"reflect"
	"strings"

	corev1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"

	"k8s.libre.sh/interfaces"
	"sigs.k8s.io/controller-runtime/pkg/client"
)

func Marshal(i interface{}) (Parameters, error) {
	parameters := Parameters{}

	t := reflect.TypeOf(i)

	ifv := reflect.ValueOf(i)

	for ii := 0; ii < t.NumField(); ii++ {
		f := t.Field(ii)

		fv := ifv.Field(ii)

		parameter, ok := fv.Interface().(Parameter)
		if !ok {
			return parameters, errors.New("not parameters type")
		}

		env, ok := f.Tag.Lookup("env")
		if !ok {
			return parameters, errors.New("missing evnv tag")
		}

		if len(parameter.Key) == 0 {
			parameter.Key = env
		}

		parameters = append(parameters, &parameter)
	}

	return parameters, nil
}

// Sort parameters by keys
func ParametersByKey(p *Parameters) map[string]*Parameter {
	paramsByKey := map[string]*Parameter{}

	for _, param := range *p {
		paramsByKey[param.Key] = param
	}
	return paramsByKey
}

// Transform parameters to key:value
func KeyPairValues(p *Parameters) map[string]string {
	data := make(map[string]string)

	for _, param := range *p {
		data[param.Key] = param.Value
	}

	return data
}

func MergeParameters(dest, src *Parameters) *Parameters {

	params := Parameters{}

	srcByKey := ParametersByKey(src)
	destByKey := ParametersByKey(dest)

	for k, param := range destByKey {
		if srcByKey[k] != nil {
			param = srcByKey[k]
			delete(srcByKey, k)
		}
		params = append(params, param)
	}

	for _, param := range srcByKey {
		params = append(params, param)
	}

	return &params
}

func MergeParametersFromResource(r interfaces.Reconcile, obj, owner interfaces.Object, params Parameters) (Parameters, error) {

	//	ps := &Parameters{}

	data, err := GetDataFromResource(r, obj, owner)
	if err != nil {
		return nil, err
	}

	ps := MergeParametersWithData(params, data)

	return ps, nil
}

func MergeParametersWithData(params Parameters, data map[string]string) Parameters {

	paramsByKey := ParametersByKey(&params)

	if len(data) > 0 {
		if len(params) > 0 {
			for _, p := range params {
				paramsByKey[p.Key].Value = string(data[p.FromKey])
				// Reset default Generate
				paramsByKey[p.Key].Generate = ""
			}
		} else {
			for k, v := range data {
				if paramsByKey[k] == nil {
					paramsByKey[k] = &Parameter{}

				}
				paramsByKey[k].Value = v
				paramsByKey[k].Key = k

				// TODO TO FIX This should be checked
				//	paramsByKey[kk].Type = pType

				// Reset default Generate
				paramsByKey[k].Generate = ""
			}
		}
	}

	// TODO TO FIX
	ps := Parameters{}

	for _, v := range paramsByKey {
		ps = append(ps, v)
	}

	return ps

}

func GetDataFromResource(r interfaces.Reconcile, obj, owner interfaces.Object) (data map[string]string, err error) {

	objData := make(map[string]string)
	data = make(map[string]string)

	objectKey, err := client.ObjectKeyFromObject(obj)
	if err != nil {
		return data, err
	}

	err = r.GetClient().Get(context.Background(), objectKey, obj)
	if err != nil {
		return data, err
	}

	switch obj.GetObjectKind().GroupVersionKind().Kind {
	case "Secret":
		for k, v := range obj.(*corev1.Secret).Data {
			objData[k] = string(v)
		}
	case "ConfigMap":
		objData = obj.(*corev1.ConfigMap).Data
	default:
		return data, errors.New("object kind should be ConfigMap or Secret")
	}

	if metav1.IsControlledBy(obj, owner) {
		if len(obj.GetAnnotations()["settings.k8s.libre.sh/generate"]) > 0 {
			genParams := strings.Split(obj.GetAnnotations()["settings.k8s.libre.sh/generate"], ",")
			for _, param := range genParams {
				data[param] = objData[param]
			}
		} else {
			data = objData
		}
	}

	return data, nil
}
