/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package deployment

import (
	interfaces "k8s.libre.sh/interfaces"
	meta "k8s.libre.sh/meta"
	pod "k8s.libre.sh/objects/pod"

	appsv1 "k8s.io/api/apps/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// Deployment is the specification of the desired behavior of the Deployment.
// It is a stripped down version of https://godoc.org/k8s.io/api/apps/v1#Deployment
// with only user definied specs
//
// +kubebuilder:object:generate=true
type Deployment struct {
	// Standard object metadata.
	// +optional
	*meta.ObjectMeta `json:"meta,omitempty"`
	// Specification of the desired behavior of the Deployment.
	// +optional
	*DeploymentSpec `json:",inline"`
}

// DeploymentSpec is the specification of the desired behavior of the DeploymentSpec.
// It is a stripped down version of https://godoc.org/k8s.io/api/apps/v1#DeploymentSpec
// with only user definied specs
//
// +kubebuilder:object:generate=true
type DeploymentSpec struct {
	// Number of desired pods. This is a pointer to distinguish between explicit
	// zero and not specified. Defaults to 1.
	// +optional
	Replicas *int32 `json:"replicas,omitempty"`
	// The deployment strategy to use to replace existing pods with new ones.
	// +optional
	// +patchStrategy=retainKeys
	Strategy appsv1.DeploymentStrategy `json:"strategy,omitempty"`
	// Pod describes the pod that will be created.
	*pod.Pod `json:",inline"`
}

func (spec *DeploymentSpec) GetReplicas() *int32 {
	return spec.Replicas
}

func (spec *DeploymentSpec) GetDeploymentStrategy() appsv1.DeploymentStrategy {
	return spec.Strategy
}

func (o *Deployment) TemplateValues() map[string]interface{} {
	values := make(map[string]interface{})
	values["meta"] = o.ObjectMeta.TemplateValues()
	// values["replicas"] = strconv.FormatInt(int64(*o.Replicas), 10)
	values["replicas"] = o.Replicas
	values["strategy"] = o.Strategy
	values["pod"] = o.Pod.PodSpec.TemplateValues()
	return values
}

func (o *Deployment) GetObject() interfaces.Object {
	obj := &appsv1.Deployment{
		TypeMeta: metav1.TypeMeta{
			Kind:       "Deployment",
			APIVersion: "apps/v1",
		},
	}

	obj.SetName(o.GetName())
	obj.SetNamespace(o.GetNamespace())

	return obj
}

func (o *Deployment) Mutate(obj interfaces.Object) error {

	deploy := obj.(*appsv1.Deployment)

	// TODO TO FIX merge meta
	o.Pod.ObjectMeta = o.ObjectMeta

	err := MutateDeployment(o, deploy)
	if err != nil {
		return err
	}

	err = meta.MutateMeta(o.ObjectMeta, obj)
	if err != nil {
		return err
	}

	deploy.Spec.Template.ObjectMeta.Name = deploy.ObjectMeta.Name

	deploy.Spec.Template.ObjectMeta.Labels = o.ObjectMeta.Labels

	deploy.Spec.Selector = metav1.SetAsLabelSelector(o.ObjectMeta.Labels)

	return nil
}

func (o *Deployment) GetObjectKind() string  { return "Deployment" }
func (o *Deployment) GetObjectGroup() string { return "apps" }
