/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package status

import (
	corev1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// ApplicationStatus defines controller's the observed state of Application
// +kubebuilder:object:generate=true
type ApplicationStatus struct {
	// ObservedGeneration is the most recent generation observed. It corresponds to the
	// Object's generation, which is updated on mutation by the API Server.
	// +optional
	ObservedGeneration int64 `json:"observedGeneration,omitempty" protobuf:"varint,1,opt,name=observedGeneration"`
	// Conditions represents the latest state of the object
	// +optional
	// +patchMergeKey=type
	// +patchStrategy=merge
	Conditions []Condition `json:"conditions,omitempty" patchStrategy:"merge" patchMergeKey:"type" protobuf:"bytes,10,rep,name=conditions"`
	// Resources embeds a list of object statuses
	// +optional
	Components map[string]*ComponentStatus `json:",inline,omitempty"`
	// ComponentsReady: status of the components in the format ready/total
	// +optional
	ComponentsReady string `json:"componentsReady,omitempty"`
}

func (app *ApplicationStatus) GetResources() *Resources {
	resources := &Resources{}

	for _, c := range app.Components {
		resources.Objects = append(resources.Objects, c.GetObjects()...)
	}

	return resources
}

// ApplicationStatus defines controller's the observed state of Application
// +kubebuilder:object:generate=true
type ComponentStatus struct {
	// Resources embeds a list of object statuses
	// +optional
	Resources `json:",inline,omitempty"`
	// ComponentsReady: status of the components in the format ready/total
	// +optional
	ComponentsReady string `json:"componentsReady,omitempty"`
}

// Condition describes the state of an object at a certain point.
// +kubebuilder:object:generate=true
type Condition struct {
	// Type of condition.
	Type ConditionType `json:"type" protobuf:"bytes,1,opt,name=type,casttype=StatefulSetConditionType"`
	// Status of the condition, one of True, False, Unknown.
	Status corev1.ConditionStatus `json:"status" protobuf:"bytes,2,opt,name=status,casttype=k8s.io/api/core/v1.ConditionStatus"`
	// The reason for the condition's last transition.
	// +optional
	Reason string `json:"reason,omitempty" protobuf:"bytes,4,opt,name=reason"`
	// A human readable message indicating details about the transition.
	// +optional
	Message string `json:"message,omitempty" protobuf:"bytes,5,opt,name=message"`
	// Last time the condition was probed
	// +optional
	LastUpdateTime metav1.Time `json:"lastUpdateTime,omitempty" protobuf:"bytes,3,opt,name=lastProbeTime"`
	// Last time the condition transitioned from one status to another.
	// +optional
	LastTransitionTime metav1.Time `json:"lastTransitionTime,omitempty" protobuf:"bytes,3,opt,name=lastTransitionTime"`
}

// ComponentList is a generic status holder for the top level resource
// +kubebuilder:object:generate=true
type Resources struct {
	// Object status array for all matching objects
	Objects []ObjectStatus `json:"objects,omitempty"`
}

func Difference(original, desired *Resources) *Resources {
	// diff := original.DeepCopy()

	diff := &Resources{}
	diffByKey := make(map[string]ObjectStatus, len(original.Objects))
	// originalByKey := make(map[string]ObjectStatus, len(original.Objects))
	desiredByKey := make(map[string]ObjectStatus, len(desired.Objects))

	for _, obj := range original.Objects {
		diffByKey[obj.Link] = obj
	}

	for _, obj := range desired.Objects {
		desiredByKey[obj.Link] = obj
	}

	for item := range desiredByKey {
		delete(diffByKey, item)
	}

	for _, obj := range diffByKey {
		diff.Objects = append(diff.Objects, obj)
	}

	return diff
}

func (res *Resources) GetObjects() []ObjectStatus {
	return res.Objects
}

// ObjectStatus is a generic status holder for objects
// +kubebuilder:object:generate=true
type ObjectStatus struct {
	// Link to object
	Link string `json:"link,omitempty"`
	// Name of object
	Name string `json:"name,omitempty"`
	// Kind of object
	Kind string `json:"kind,omitempty"`
	// Object group
	Group string `json:"group,omitempty"`
	// Status. Values: InProgress, Ready, Unknown
	Status string `json:"status,omitempty"`
}

// ConditionType encodes information on the condition
// +kubebuilder:object:generate=true
type ConditionType string
